/* GtkX10  - GUI controller for the CM17A Firecracker kit from X10
 *
 * config-win.c - Configuration window.
 *
 *  Copyright (C) 1999  Chris Edwards <cedwards@naxs.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config-win.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

_Config_	*Config;
_ConfigWin_	*ConfigWin;

void save_config()
{
	char *env;
	char string[255];
	char string2[9];
	char *name;
	FILE *config;
	
	env = getenv("HOME");
	strcpy(string,env);
	strcpy(string2,"/.gtkx10");
	strcat(string, string2);
	name = string;
	config = fopen(name, "w+");
	if (config != NULL)
	{
		fprintf(config, Config->device);
		fprintf(config, "\n");
		fprintf(config, "%d\n",Config->bitwait);
		fprintf(config, "%d",Config->delay);
		fclose (config);
		Config->status = 0;
	}
}

void read_config()
{
	char *env;
	char string[255];
	char string2[9];
	char *name;
	char tmp[11];
	FILE *config;
	
	prepare_config();
	env = getenv("HOME");
	strcpy(string,env);
	strcpy(string2,"/.gtkx10");
	strcat(string, string2);
	name = string;
	
	if ((config = fopen(name, "r+")) == NULL)
	{
		g_print("Config file not found! Setting Default Values!\n");
		Config->device = "/dev/ttyS0";
		Config->bitwait = 5;
		Config->delay = 100000;
		Config->status = -1;
	}
	else
	{	
		Config->status = 0;
		fgets(tmp, sizeof(tmp), config);
		Config->device = (char *) malloc(sizeof(tmp));
		sprintf(Config->device,"%s", tmp);	
		// why?? // 
fgets(tmp,sizeof(tmp),config);
	
		fgets(tmp, sizeof(tmp), config);
		Config->bitwait = atoi(tmp);
	
		fgets(tmp, sizeof(tmp), config);
		Config->delay = atoi(tmp);
	
		fclose (config);
	}
}

/* itoa taken from K&R */
char * itoa (int n, char s[])
{
	int i, j, sign, x;

	if ((sign = n) < 0)
		n = -n;
	i = 0;
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	for (i=0, j=strlen(s)-1; i < j; i++, j--)
	{
		x = s[i];
		s[i] = s[j];
		s[j] = x;
	} 
	return (s); 
}

void set_config_window_data()
{
	char tmp[30];
	prepare_config();
	gtk_entry_set_text(GTK_ENTRY(ConfigWin->device), Config->device);  
	gtk_entry_set_text(GTK_ENTRY(ConfigWin->bitwait), itoa(Config->bitwait,tmp) );
	gtk_entry_set_text(GTK_ENTRY(ConfigWin->delay), itoa(Config->delay,tmp) );  
}

void cancel_cb(GtkWidget *widget, gpointer window)
{
	gtk_widget_hide(window);
}

void ok_cb(GtkWidget *widget, gpointer window)
{
	char *Test;
	Test= gtk_entry_get_text(GTK_ENTRY(ConfigWin->device));
	Config->device = Test;
	Test= gtk_entry_get_text(GTK_ENTRY(ConfigWin->bitwait));
	Config->bitwait= atoi(Test);
	Test = gtk_entry_get_text(GTK_ENTRY(ConfigWin->delay));
	Config->delay = atoi(Test);
	save_config();
	gtk_widget_hide(ConfigWin->window);
}

void prepare_config()
{
	GtkWidget *label;
	
	/* Set up the Config Window */
	ConfigWin->window = gtk_window_new (GTK_WINDOW_DIALOG);
	gtk_widget_set_usize(GTK_WIDGET(ConfigWin->window),150, 100);
	gtk_window_set_title(GTK_WINDOW(ConfigWin->window), "Configuration");
	gtk_window_set_policy(GTK_WINDOW(ConfigWin->window),1,1,0);
	gtk_window_set_position(GTK_WINDOW(ConfigWin->window),1);
		
	ConfigWin->vbox = gtk_vbox_new(TRUE,0);
	gtk_container_add(GTK_CONTAINER(ConfigWin->window), ConfigWin->vbox);
	gtk_widget_show(ConfigWin->vbox);
	
	ConfigWin->device = gtk_entry_new_with_max_length(11);
	label = gtk_label_new("Device: ");
	
	ConfigWin->hbox = gtk_hbox_new(TRUE,0);
	gtk_box_pack_start(GTK_BOX(ConfigWin->vbox), ConfigWin->hbox, FALSE, TRUE, 0);
	gtk_widget_show(ConfigWin->hbox);
	
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox), ConfigWin->device, TRUE, TRUE, 0);
	
	gtk_widget_show(label);
	gtk_widget_show(ConfigWin->device);
	
	ConfigWin->bitwait = gtk_entry_new_with_max_length(11);
	label = gtk_label_new("Bit Wait: ");
	
	ConfigWin->hbox = gtk_hbox_new(TRUE,0);
	gtk_box_pack_start(GTK_BOX(ConfigWin->vbox), ConfigWin->hbox, FALSE, TRUE, 0);
	gtk_widget_show(ConfigWin->hbox);
	
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox), ConfigWin->bitwait, TRUE, TRUE, 0);
	
	gtk_widget_show(label);
	gtk_widget_show(ConfigWin->bitwait);
	
	ConfigWin->delay = gtk_entry_new_with_max_length(11);
	label = gtk_label_new("Delay: ");
	
	ConfigWin->hbox = gtk_hbox_new(TRUE,0);
	gtk_box_pack_start(GTK_BOX(ConfigWin->vbox), ConfigWin->hbox, FALSE, TRUE, 0);
	gtk_widget_show(ConfigWin->hbox);
	
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox), ConfigWin->delay, TRUE, TRUE, 0);
	
	gtk_widget_show(label);
	gtk_widget_show(ConfigWin->delay);
	
	ConfigWin->hbox = gtk_hbox_new(TRUE,0);
	gtk_box_pack_start(GTK_BOX(ConfigWin->vbox), ConfigWin->hbox, FALSE, TRUE, 0);
	gtk_widget_show(ConfigWin->hbox);
	
	
	/* Buttons */
	ConfigWin->ok = gtk_button_new_with_label("OK");
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox), ConfigWin->ok, FALSE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(ConfigWin->ok), "clicked", GTK_SIGNAL_FUNC(ok_cb),ConfigWin->window);
	gtk_widget_show(ConfigWin->ok);
	
	ConfigWin->cancel = gtk_button_new_with_label("Cancel");
	gtk_box_pack_start(GTK_BOX(ConfigWin->hbox),ConfigWin->cancel, FALSE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(ConfigWin->cancel), "clicked", GTK_SIGNAL_FUNC(cancel_cb),ConfigWin->window);
	gtk_widget_show(ConfigWin->cancel);
}	
