/* GtkX10  - GUI controller for the CM17A Firecracker kit from X10
 *
 * gtkx10.c - Main GUI and device calls
 *
 *  Copyright (C) 1999  Chris Edwards <cedwards@naxs.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Added label file functionality 2/1/2001 qarce@engr.sgi.com
 *       Added variables:
 *         static char *curr_unit
 *         static char *button_label_sets[]
 *         static GtkWidget *gWidget
 *
 *       Added functions:
 *         void clean_mem(void)
 *         int load_button_labels(void)            
 *
 */   

#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "br_cmd.h"
#include "config-win.h"

_ConfigWin_	*ConfigWin;
_Config_	*Config;

#define NUMBER_GROUPS 15	/* A-O */	
#define NUMBER_DEVICES 16	/* 1-16 */

void exec_cmd(char *devname, int cmd);
void build_button_table(GtkWidget *hbox, char group);


void config_cb(GtkWidget *widget, gpointer data)
{
	gtk_widget_show(ConfigWin->window);
}

static GtkItemFactoryEntry menu_items[] =
{
	{"/_Config",       "<control>C", config_cb, 0, NULL},
	{"/_Quit",       "<control>Q", gtk_main_quit, 0, NULL},
};

static char *curr_unit= "A"; /* default unit */
static char *button_label_sets[NUMBER_GROUPS][NUMBER_DEVICES];	/* storage for button labels */
static GtkWidget *gWidget; /* used to keep reference to buttons so they can be re-built */

void get_main_menu(GtkWidget *window, GtkWidget ** menubar)
{
	int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	accel_group = gtk_accel_group_new();
	/* This function initializes the item factory.
	Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
	or GTK_TYPE_OPTION_MENU.
	Param 2: The path of the menu.
	Param 3: A pointer to a gtk_accel_group.  The item factory sets up
	the accelerator table while generating menus. */

	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>",accel_group);

	/* This function generates the menu items.  Pass the item factory, the
	number of items in the array, the array itself, and any callback data
	for the the menu items. */
	gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);

	/* Attach the new accelerator group to the window. */
	gtk_accel_group_attach (accel_group, GTK_OBJECT (window));

	if (menubar)
	/* Finally, return the actual menu bar created by the item factory. */
	*menubar = gtk_item_factory_get_widget(item_factory, "<main>");
}

/* function to parse the command to be sent to the device */
int get_cmd(char *arg)
{
	if (!strcasecmp("on", arg)) {
		return ON;
	} else if (!strcasecmp("off", arg)) {
		return OFF;
	} else if (!strcasecmp("dim", arg)) {
		return DIM;
	} else if (!strcasecmp("bright", arg)) {
		return BRIGHT;
	}

	return -1;
}

/* function to parse the numerical part of the device name */
int get_device(char *arg, char cmd)
{
	char device;


	/*
	 * Figure out the numeric part of the address of a unit (which device
	 *  in a group) we want to talk to
	 *
	 * If the command is a "dim" command, the device should be set to 0.
	 *  Unfortunately, some of the address lines are used for the dim/
	 *  bright commands.
	 */

	if ((cmd == DIM) || (cmd == BRIGHT)) {
                if (strlen(arg) != 1)
			return -1;
		device = 0;
	} else {

		if (strlen(arg) < 2)
			return -1;
		device = (char)atoi(&arg[1]) - 1;
		if ((device < 0) || (device > 15))
			return -1;
	}

	return device;
}

/* function to parse out the group from the device */
int get_group(char *arg)
{
	/*
	 * Get the letter part of a unit address (what "group" of units 
	 *  requested one is in)
	 */

	int group;


	if (!strlen(arg))
		return -1;

	group = toupper(arg[0]) - 'A';
	if ((group > 15) || (group < 0))
		return -1;

	return group;
}

/* radio button callback */
void radio_cb(GtkWidget *widget, gpointer data)
{
	GtkWidget *tWidget;
	gboolean active;
	active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	if (active == 1)
	{
		curr_unit = data;
	}
	
	/* kill the set of buttons and re-build it */
	tWidget = gWidget->parent;
	gtk_widget_destroy(gWidget);
	build_button_table(tWidget, *(char *)data);
}

/* On button click callback */
void on_clicked_cb(GtkWidget *widget, char *data)
{
	char string1[2];
	char string2[3];
	
	/* Get the device unit and number and put them together 
	 * It's a rather crude hack that I will improve later */
	strcpy(string1,data);
	strcpy(string2,curr_unit);
	strcat(string2,string1);
	exec_cmd(string2,ON);
}

/* Off button click callback */
void off_clicked_cb(GtkWidget *widget, char *data)
{
	char string1[2];
	char string2[3];
	
	/* Get the device unit and number and put them together 
	 * It's a rather crude hack that I will improve later */
	strcpy(string1,data);
	strcpy(string2,curr_unit);
	strcat(string2,string1);
	exec_cmd(string2,OFF);
}

/* dim switch callback */
void light_clicked_cb(GtkWidget *widget, int data)
{
	exec_cmd(curr_unit,data);
}

/* function to ease creation of on/off switches */
void init_button(GtkWidget *table, int num, int next_num, char *label, char *data)
{
	GtkWidget *box;
	GtkWidget *onbutton;
	GtkWidget *offbutton;
	box = gtk_hbox_new(FALSE,0);
	gtk_container_set_border_width (GTK_CONTAINER (box), 3);
	gtk_table_attach_defaults (GTK_TABLE(table), box, 0, 1, num, next_num);
	onbutton=gtk_button_new_with_label(label);
	offbutton=gtk_button_new_with_label(label);
	gtk_signal_connect(GTK_OBJECT(onbutton),"clicked",GTK_SIGNAL_FUNC(on_clicked_cb), (gpointer)data);
	gtk_signal_connect(GTK_OBJECT(offbutton),"clicked",GTK_SIGNAL_FUNC(off_clicked_cb),(gpointer)data);
	gtk_box_pack_start(GTK_BOX(box),onbutton,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(box),offbutton,TRUE,TRUE,0);
	gtk_widget_show(onbutton);
	gtk_widget_show(offbutton);
	gtk_widget_show(box);
}

/* funtion to create the dimming siwtches */
void dim_buttons(GtkWidget *table, int num, int next_num)
{
	GtkWidget *box;
	GtkWidget *upbutton;
	GtkWidget *downbutton;
	GtkWidget *arrow;
	
	box = gtk_hbox_new(FALSE,0);
	gtk_container_set_border_width (GTK_CONTAINER (box), 3);
	gtk_table_attach_defaults (GTK_TABLE(table), box, 0, 1, num, next_num);
	upbutton=gtk_button_new();
	arrow = gtk_arrow_new(GTK_ARROW_UP, GTK_SHADOW_OUT);
	gtk_container_add (GTK_CONTAINER (upbutton), arrow);
	gtk_signal_connect(GTK_OBJECT(upbutton),"clicked",GTK_SIGNAL_FUNC(light_clicked_cb), (gpointer)BRIGHT);
	gtk_box_pack_start(GTK_BOX(box),upbutton,TRUE,TRUE,0);
	gtk_widget_show(arrow);
	gtk_widget_show(upbutton);
	
	downbutton=gtk_button_new();
	arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
	gtk_container_add (GTK_CONTAINER (downbutton), arrow);
	gtk_signal_connect(GTK_OBJECT(downbutton),"clicked",GTK_SIGNAL_FUNC(light_clicked_cb),(gpointer)DIM);
	gtk_box_pack_start(GTK_BOX(box),downbutton,TRUE,TRUE,0);
	gtk_widget_show(arrow);
	gtk_widget_show(downbutton);
	
	gtk_widget_show(box);
}

/* function to create the C through O toggle buttons */
void toggles(GtkWidget *box, GtkWidget *unit, gchar *label)
{
	unit = gtk_radio_button_new_with_label(gtk_radio_button_group (GTK_RADIO_BUTTON (unit)),label);
	gtk_box_pack_start (GTK_BOX (box), unit, TRUE, TRUE, 0);
	gtk_widget_show (unit);
	gtk_signal_connect(GTK_OBJECT(unit),"toggled",GTK_SIGNAL_FUNC(radio_cb),label);
}

/*
 *  Load the button labels from file ~/.gtkx10.labels
 *  Fill in missing labels with number.
 */
int load_button_labels(void)
{
        char *env;
	int LINE_LENGTH = 255;
        char string[LINE_LENGTH];
        char string2[LINE_LENGTH];
        char *name;
        FILE *config;
	int return_code = 0;
	char *group, *device, *label, *tmp_ptr;
	int i, tmp;

        env = getenv("HOME");
        strcpy(string,env);
        strcpy(string2,"/.gtkx10.labels");
        strcat(string, string2);
        name = string;
        config = fopen(name, "r");         


	if( config != NULL)
	{
		return_code = 1;

		for ( i = 0; fgets (string, LINE_LENGTH,  config) ; i++ )
		{	
			if ( strlen(string) > 5 )
			{ /* yep, the string is at least the min. length */
				/* pars line */
				group = strtok (string, ";");
				device = strtok (NULL, ";");
				label = strtok (NULL, ";");
		
				/* ensure we found input */
				if ( (strlen(group) == 0) || (strlen(device) == 0) || (strlen(label) == 0) )
				{
					printf("\nconfig file line %d is inproperly structured", i);
				}
				else
				{
					label[strlen(label)-1]='\0';	/* chop the '\n' */
					tmp = group[0] - 'A';		/* find the group */
					tmp_ptr = malloc(strlen(label)+1);
					sprintf(tmp_ptr,"%s", label);	/* store the label in mem. */
					button_label_sets[tmp][atoi(device) - 1]= tmp_ptr; /* add it */
				} /* End if/else an config missing */

				/* clear strings from error case. just for fun */				
				group[0]='\0'; 
				device[0]='\0';
				label[0]='\0';
			} /* End if to protect against bad line formation in labels file */
		} /* End for get lines from file */

		fclose (config);
	}
	else
	{
		printf("\nNo Label file found at: %s\nWith format: [A-O] [1-8] [String]\n\tSetting to numbers.\n", name);
		fflush(stdout);
	} /* End if/else config file found */

	/* Fill in rest of button labels with numbers. */
	for (i=0; i < NUMBER_GROUPS;i++)
	{
		for (tmp=0; tmp < NUMBER_DEVICES; tmp++)
		{
			if (button_label_sets[i][tmp] == NULL)
			{
				tmp_ptr = malloc(sizeof (char) *3);
				sprintf(tmp_ptr,"%d", tmp+1);
				button_label_sets[i][tmp] = tmp_ptr;
			}
		} /* End for NUMBER_DEVICES */
	} /* End for NUMBER_GROUPS */

	return (return_code);
} /* End int load_button_labels(void) */

/*
 * clean up the mem used
 */
void clean_mem()
{
        int i,a;

        for (a=0; a < NUMBER_GROUPS;a++)
        {
                for (i=0; i < NUMBER_DEVICES; i++)
                {
                        free (button_label_sets[a][i]);
                }
        }
	free (curr_unit);
        printf("\nFinished Cleanup\n");
}

/*
*/
void build_button_table(GtkWidget *hbox, char group)
{
	
	GtkWidget *hbox2;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *separator;
	int i;
	char *data_ptr;

	table = gtk_table_new (18, 1, TRUE);

	/* save a pointer to the box */
	gWidget = table;

	gtk_box_pack_start(GTK_BOX(hbox),table,FALSE,TRUE,0);
	gtk_widget_show(table);

	/* moved orig. data from here to before func. call */
	
	hbox2 = gtk_hbox_new(FALSE,0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox2), 3);
	gtk_table_attach_defaults (GTK_TABLE(table), hbox2, 0, 1, 0, 1);
	label = gtk_label_new("On ");
	gtk_box_pack_start(GTK_BOX(hbox2),label,TRUE,TRUE,0);
	gtk_widget_show(label);
	label = gtk_label_new("Off");
	gtk_box_pack_start(GTK_BOX(hbox2),label,TRUE,TRUE,0);
	gtk_widget_show(label);
	gtk_widget_show(hbox2);

	/* Build New Buttons */	
	for (i=0; i < NUMBER_DEVICES; i++)
	{	
		data_ptr = malloc (sizeof(char)*3);
		sprintf(data_ptr,"%d",i+1);
        	init_button(table,i+1,i+2,button_label_sets[(group - 'A')][i],data_ptr);      
	}

	i++;
	separator = gtk_hseparator_new();
	gtk_widget_set_usize (separator, 50, 5);
	gtk_table_attach_defaults (GTK_TABLE(table), separator, 0, 1, i, i+1);
	gtk_widget_show(separator);
	i++;	

	dim_buttons(table,i,i+1);

}

int main(int argc, char *argv[])
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *vbox2;
	GtkWidget *menubar;
	GtkWidget *unit;
	GtkWidget *tmp;
	GSList *unitgroup;

	
	gtk_init (&argc, &argv);
	
	ConfigWin = g_malloc(sizeof(_ConfigWin_));
	Config = g_malloc(sizeof(_Config_));
	read_config();
	
	set_config_window_data();
	load_button_labels();

	/* Set up the Main Window */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	//gtk_widget_set_usize(GTK_WIDGET(window),170, 600);
	gtk_window_set_title(GTK_WINDOW(window), "GtkX10");
	gtk_window_set_policy(GTK_WINDOW(window),1,1,1);
	gtk_window_set_position(GTK_WINDOW(window),1);
	gtk_signal_connect(GTK_OBJECT(window),"destroy",GTK_SIGNAL_FUNC(gtk_main_quit),NULL);

	vbox = gtk_vbox_new(FALSE,0);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	get_main_menu(window, &menubar);
	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);
	hbox = gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,FALSE,0);
	vbox2 = gtk_vbox_new(FALSE,0);
	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 10);
	gtk_box_pack_start(GTK_BOX(hbox),vbox2,TRUE,FALSE,0);

	gtk_widget_show(vbox2);


	build_button_table(hbox, 'A');

 	gtk_widget_show(menubar);
	gtk_widget_show(hbox);
        gtk_widget_show(vbox);       

	/* Set up house code radio boxes */
	unit = gtk_radio_button_new_with_label (NULL, "A");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (unit), TRUE);
	gtk_box_pack_start (GTK_BOX (vbox2), unit, TRUE, TRUE, 0);
	gtk_widget_show (unit);
	gtk_signal_connect(GTK_OBJECT(unit),"toggled",GTK_SIGNAL_FUNC(radio_cb),"A");

	unitgroup = gtk_radio_button_group (GTK_RADIO_BUTTON (unit));
	unit = gtk_radio_button_new_with_label(unitgroup, "B");
        /* Why Set B to default? */
        /* gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (unit), TRUE); */
	gtk_box_pack_start (GTK_BOX (vbox2), unit, TRUE, TRUE, 0);
	gtk_widget_show (unit);
	gtk_signal_connect(GTK_OBJECT(unit),"toggled",GTK_SIGNAL_FUNC(radio_cb),"B");
	tmp = unit;
	toggles(vbox2,unit,"C");
	toggles(vbox2,unit,"D");
	toggles(vbox2,unit,"E");
	toggles(vbox2,unit,"F");
	toggles(vbox2,unit,"G");
	toggles(vbox2,unit,"H");
	toggles(vbox2,unit,"I");
	toggles(vbox2,unit,"J");
	toggles(vbox2,unit,"K");
	toggles(vbox2,unit,"L");
	toggles(vbox2,unit,"M");
	toggles(vbox2,unit,"N");
	toggles(vbox2,unit,"O");	


	gtk_widget_show(window);
	if (Config->status == -1)
	{
		gtk_widget_show(ConfigWin->window);
	}

	/* Set the initial state to be group B */
	radio_cb(tmp, "A");

	gtk_main();

        /* clean up button_label_sets */ 
	clean_mem();  	
	
	return 0;

}

/* All code for actually locking the serial port and sending commands here to leave the device unlocked */
void exec_cmd(char *devname, int cmd)
{
	int fd;
	unsigned short unit;
	char group;
	char device;
	
	/* Use existing functions from bottle rocket */
	device = get_device(devname, cmd);
	group = get_group(devname);

	unit = (unsigned char)(group << 4 | device);
	
	/*
	 * Snag the serial port...
	 */

	if ((fd = open(Config->device, O_RDWR)) < 0) {
		fprintf(stderr, " Error %d opening %s.\n", errno, Config->device);
		exit(errno);
	}
	x10_br_out(fd,(unsigned char)unit,cmd);
	close(fd);
}
